@@ -3,7 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -48,7 +49,7 @@ namespace OrthancPlugins
 
   enum MetadataMode
   {
-    MetadataMode_Full,           // Read all the DICOM instances from the storage area
+    MetadataMode_Full,           // Read all the DICOM instances from the storage area and store them in an attachment on StableSeries event
     MetadataMode_MainDicomTags,  // Only use the Orthanc database (main DICOM tags only)
     MetadataMode_Extrapolate     // Extrapolate user-specified tags from a few DICOM instances
   };
@@ -139,5 +140,7 @@ namespace OrthancPlugins
     void SaveDicomWebServers();
 
     unsigned int GetMetadataWorkerThreadsCount();
+
+    bool IsMetadataCacheEnabled();
   }
 }
