@@ -3,7 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -919,8 +920,8 @@ void StowClient(OrthancPluginRestOutput* output,
   Json::Value resourcesForJobContent;
   ParseStowRequest(instances, httpHeaders, resourcesForJobContent, body);
 
-  OrthancPlugins::LogInfo("Sending " + boost::lexical_cast<std::string>(instances.size()) +
-                          " instances using STOW-RS to DICOMweb server: " + serverName);
+  LOG(INFO) << "Sending " << boost::lexical_cast<std::string>(instances.size()) <<
+               " instances using STOW-RS to DICOMweb server: " << serverName;
 
   std::unique_ptr<StowClientJob> job(new StowClientJob(serverName, instances, httpHeaders, resourcesForJobContent));
 
