@@ -3,7 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -40,7 +41,6 @@
 #define ORTHANC_CORE_MINIMAL_MINOR     11
 #define ORTHANC_CORE_MINIMAL_REVISION  0
 
-
 static const char* const HAS_DELETE = "HasDelete";
 
 
@@ -403,8 +403,7 @@ void DeleteClient(OrthancPluginRestOutput* output,
 static bool DisplayPerformanceWarning(OrthancPluginContext* context)
 {
   (void) DisplayPerformanceWarning;   // Disable warning about unused function
-  OrthancPluginLogWarning(context, "Performance warning in DICOMweb: "
-                          "Non-release build, runtime debug assertions are turned on");
+  LOG(WARNING) << "Performance warning in DICOMweb: Non-release build, runtime debug assertions are turned on";
   return true;
 }
 
@@ -497,16 +496,18 @@ extern "C"
 {
   ORTHANC_PLUGINS_API int32_t OrthancPluginInitialize(OrthancPluginContext* context)
   {
-    assert(DisplayPerformanceWarning(context));
+    OrthancPlugins::SetGlobalContext(context, ORTHANC_DICOM_WEB_NAME);
 
-    OrthancPlugins::SetGlobalContext(context);
-    
-#if ORTHANC_FRAMEWORK_VERSION_IS_ABOVE(1, 7, 2)
+#if ORTHANC_FRAMEWORK_VERSION_IS_ABOVE(1, 12, 4)
+    Orthanc::Logging::InitializePluginContext(context, ORTHANC_DICOM_WEB_NAME);
+#elif ORTHANC_FRAMEWORK_VERSION_IS_ABOVE(1, 7, 2)
     Orthanc::Logging::InitializePluginContext(context);
 #else
     Orthanc::Logging::Initialize(context);
 #endif
 
+    assert(DisplayPerformanceWarning(context));
+
     Orthanc::Logging::EnableInfoLevel(true);
 
     /* Check the version of the Orthanc core */
@@ -518,7 +519,7 @@ extern "C"
               ORTHANC_PLUGINS_MINIMAL_MAJOR_NUMBER,
               ORTHANC_PLUGINS_MINIMAL_MINOR_NUMBER,
               ORTHANC_PLUGINS_MINIMAL_REVISION_NUMBER);
-      OrthancPluginLogError(context, info);
+      LOG(ERROR) << info;
       return -1;
     }
 
@@ -532,7 +533,7 @@ extern "C"
               ORTHANC_CORE_MINIMAL_MAJOR,
               ORTHANC_CORE_MINIMAL_MINOR,
               ORTHANC_CORE_MINIMAL_REVISION);
-      OrthancPluginLogError(context, info);
+      LOG(ERROR) << info;
       return -1;
     }
 
@@ -548,7 +549,7 @@ extern "C"
                  << "Orthanc SDK <= 1.12.0. Retrieving metadata will be slower.";
 #endif
 
-    OrthancPluginSetDescription(context, "Implementation of DICOMweb (QIDO-RS, STOW-RS and WADO-RS) and WADO-URI.");
+    OrthancPlugins::SetDescription(ORTHANC_DICOM_WEB_NAME, "Implementation of DICOMweb (QIDO-RS, STOW-RS and WADO-RS) and WADO-URI.");
 
     try
     {
@@ -561,7 +562,7 @@ extern "C"
         std::string root = OrthancPlugins::Configuration::GetDicomWebRoot();
         assert(!root.empty() && root[root.size() - 1] == '/');
 
-        OrthancPlugins::LogWarning("URI to the DICOMweb REST API: " + root);
+        LOG(WARNING) << "URI to the DICOMweb REST API: " << root;
 
         OrthancPlugins::ChunkedRestRegistration<
           SearchForStudies /* TODO => Rename as QIDO-RS */,
@@ -635,43 +636,42 @@ extern "C"
           dictionary["DICOMWEB_ROOT"] = root.substr(1, root.size() - 2);  // Remove heading and trailing slashes
           std::string configured = Orthanc::Toolbox::SubstituteVariables(explorer, dictionary);
 
-          OrthancPluginExtendOrthancExplorer(OrthancPlugins::GetGlobalContext(), configured.c_str());
+          OrthancPlugins::ExtendOrthancExplorer(ORTHANC_DICOM_WEB_NAME, configured);
         }
         
         
         std::string uri = root + "app/client/index.html";
-        OrthancPluginSetRootUri(context, uri.c_str());
+        OrthancPlugins::SetRootUri(ORTHANC_DICOM_WEB_NAME, uri.c_str());
 
         std::string publicUrlRoot = OrthancPlugins::Configuration::GetPublicRoot();
-        OrthancPlugins::LogWarning("DICOMWeb PublicRoot: " + publicUrlRoot);
+        LOG(WARNING) << "DICOMWeb PublicRoot: " << publicUrlRoot;
       }
       else
       {
-        OrthancPlugins::LogWarning("DICOMweb support is disabled");
+        LOG(WARNING) << "DICOMweb support is disabled";
       }
 
       // Configure the WADO callback
       if (OrthancPlugins::Configuration::GetBooleanValue("EnableWado", true))
       {
         std::string wado = OrthancPlugins::Configuration::GetWadoRoot();
-        OrthancPlugins::LogWarning("URI to the WADO-URI API: " + wado);
+        LOG(WARNING) << "URI to the WADO-URI API: " << wado;
 
         OrthancPlugins::RegisterRestCallback<WadoUriCallback>(wado, true);
       }
       else
       {
-        OrthancPlugins::LogWarning("WADO-URI support is disabled");
+        LOG(WARNING) << "WADO-URI support is disabled";
       }
     }
     catch (Orthanc::OrthancException& e)
     {
-      OrthancPlugins::LogError("Exception while initializing the DICOMweb plugin: " + 
-                               std::string(e.What()));
+      LOG(ERROR) << "Exception while initializing the DICOMweb plugin: " << e.What();
       return -1;
     }
     catch (...)
     {
-      OrthancPlugins::LogError("Exception while initializing the DICOMweb plugin");
+      LOG(ERROR) << "Exception while initializing the DICOMweb plugin";
       return -1;
     }
 
@@ -686,7 +686,7 @@ extern "C"
 
   ORTHANC_PLUGINS_API const char* OrthancPluginGetName()
   {
-    return "dicom-web";
+    return ORTHANC_DICOM_WEB_NAME;
   }
 
 
