@@ -2,7 +2,9 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2021 Osimis S.A., Belgium
+ * Copyright (C) 2017-2023 Osimis S.A., Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -38,7 +40,7 @@
 
 namespace
 {
-  class ModuleMatcher
+  class ModuleMatcher : public boost::noncopyable
   {
   public:
     typedef std::map<Orthanc::DicomTag, std::string>  Filters;
@@ -66,7 +68,7 @@ namespace
           result.insert(Orthanc::DicomTag(0x0008, 0x0030));  // Study Time
           result.insert(Orthanc::DicomTag(0x0008, 0x0050));  // Accession Number
           result.insert(Orthanc::DicomTag(0x0008, 0x0056));  // Instance Availability
-          //result.insert(Orthanc::DicomTag(0x0008, 0x0061));  // Modalities in Study  => SPECIAL CASE
+          result.insert(Orthanc::DicomTag(0x0008, 0x0061));  // Modalities in Study  => SPECIAL CASE
           result.insert(Orthanc::DicomTag(0x0008, 0x0090));  // Referring Physician's Name
           result.insert(Orthanc::DicomTag(0x0008, 0x0201));  // Timezone Offset From UTC
           //result.insert(Orthanc::DicomTag(0x0008, 0x1190));  // Retrieve URL  => SPECIAL CASE
@@ -76,8 +78,8 @@ namespace
           result.insert(Orthanc::DicomTag(0x0010, 0x0040));  // Patient's Sex
           result.insert(Orthanc::DicomTag(0x0020, 0x000D));  // Study Instance UID
           result.insert(Orthanc::DicomTag(0x0020, 0x0010));  // Study ID
-          //result.insert(Orthanc::DicomTag(0x0020, 0x1206));  // Number of Study Related Series  => SPECIAL CASE
-          //result.insert(Orthanc::DicomTag(0x0020, 0x1208));  // Number of Study Related Instances  => SPECIAL CASE
+          result.insert(Orthanc::DicomTag(0x0020, 0x1206));  // Number of Study Related Series  => SPECIAL CASE
+          result.insert(Orthanc::DicomTag(0x0020, 0x1208));  // Number of Study Related Instances  => SPECIAL CASE
           break;
 
         case Orthanc::ResourceType_Series:
@@ -89,7 +91,7 @@ namespace
           //result.insert(Orthanc::DicomTag(0x0008, 0x1190));  // Retrieve URL  => SPECIAL CASE
           result.insert(Orthanc::DicomTag(0x0020, 0x000E));  // Series Instance UID
           result.insert(Orthanc::DicomTag(0x0020, 0x0011));  // Series Number
-          //result.insert(Orthanc::DicomTag(0x0020, 0x1209));  // Number of Series Related Instances  => SPECIAL CASE
+          result.insert(Orthanc::DicomTag(0x0020, 0x1209));  // Number of Series Related Instances  => SPECIAL CASE
           result.insert(Orthanc::DicomTag(0x0040, 0x0244));  // Performed Procedure Step Start Date
           result.insert(Orthanc::DicomTag(0x0040, 0x0245));  // Performed Procedure Step Start Time
           result.insert(Orthanc::DicomTag(0x0040, 0x0275));  // Request Attribute Sequence
@@ -200,7 +202,7 @@ namespace
         }
       }
 
-      OrthancPlugins::LogInfo("Arguments of QIDO-RS request:" + args);
+      LOG(INFO) << "Arguments of QIDO-RS request:" << args;
     }
 
     unsigned int GetLimit() const
@@ -272,17 +274,18 @@ namespace
         result["CaseSensitive"] = caseSensitive;
       }
 
-      result["Expand"] = false;
+      result["Expand"] = true;
+      result["Full"] = true;
       result["Query"] = Json::objectValue;
       result["Limit"] = limit_;
       result["Since"] = offset_;
+      result["RequestedTags"] = Json::arrayValue;
 
       if (offset_ != 0 &&
           !OrthancPlugins::CheckMinimalOrthancVersion(1, 3, 0))
       {
-        OrthancPlugins::LogError(
-          "QIDO-RS request with \"offset\" argument: "
-          "Only available if the Orthanc core version is >= 1.3.0");
+        LOG(ERROR) << "QIDO-RS request with \"offset\" argument: "
+                      "Only available if the Orthanc core version is >= 1.3.0";
       }
       
       for (Filters::const_iterator it = filters_.begin(); 
@@ -290,130 +293,22 @@ namespace
       {
         result["Query"][it->first.Format()] = it->second;
       }
+
+      std::set<Orthanc::DicomTag> requestedTags;
+      ExtractResultFields(requestedTags, level);
+
+      for (std::set<Orthanc::DicomTag>::const_iterator it = requestedTags.begin();
+          it != requestedTags.end(); ++it)
+      {
+        result["RequestedTags"].append(it->Format());
+      }
     }
 
 
-    void ComputeDerivedTags(Orthanc::DicomMap& target,
-                            std::string& someInstance,
-                            Orthanc::ResourceType level,
-                            const std::string& resource) const
+
+    void ExtractResultFields(std::set<Orthanc::DicomTag>& fields,
+                             Orthanc::ResourceType level) const
     {
-      static const char* const INSTANCES = "Instances";      
-      static const char* const MAIN_DICOM_TAGS = "MainDicomTags";
-      static const char* const MODALITY = "Modality";
-      
-      switch (level)
-      {
-        case Orthanc::ResourceType_Instance:
-          someInstance = resource;
-          break;
-
-        case Orthanc::ResourceType_Study:
-        {
-          Json::Value series;
-          if (OrthancPlugins::RestApiGet(series, "/studies/" + resource + "/series?expand", false) &&
-              series.type() == Json::arrayValue)
-          {
-            // Collect the Modality of all the child series, and 
-            std::set<std::string> modalities;
-            unsigned int countInstances = 0;
-            
-            for (Json::Value::ArrayIndex i = 0; i < series.size(); i++)
-            {
-              if (series[i].type() == Json::objectValue)
-              {
-                if (series[i].isMember(MAIN_DICOM_TAGS) &&
-                    series[i][MAIN_DICOM_TAGS].type() == Json::objectValue &&
-                    series[i][MAIN_DICOM_TAGS].isMember(MODALITY) &&
-                    series[i][MAIN_DICOM_TAGS][MODALITY].type() == Json::stringValue)
-                {
-                  modalities.insert(series[i][MAIN_DICOM_TAGS][MODALITY].asString());
-                }
-                
-                if (series[i].isMember(INSTANCES) &&
-                    series[i][INSTANCES].type() == Json::arrayValue)
-                {
-                  if (series[i][INSTANCES].size() > 0 &&
-                      series[i][INSTANCES][0].type() == Json::stringValue)
-                  {
-                    someInstance = series[i][INSTANCES][0].asString();
-                  }
-                  
-                  countInstances += series[i][INSTANCES].size();
-                }
-              }
-            }
-
-            target.SetValue(Orthanc::DICOM_TAG_NUMBER_OF_STUDY_RELATED_SERIES,
-                            boost::lexical_cast<std::string>(series.size()), false);
-            
-            target.SetValue(Orthanc::DICOM_TAG_NUMBER_OF_STUDY_RELATED_INSTANCES,
-                            boost::lexical_cast<std::string>(countInstances), false);
-            
-            std::string s;
-            for (std::set<std::string>::const_iterator 
-                   it = modalities.begin(); it != modalities.end(); ++it)
-            {
-              if (!s.empty())
-              {
-                s += "\\";
-              }
-
-              s += *it;
-            }
-
-            target.SetValue(Orthanc::DICOM_TAG_MODALITIES_IN_STUDY, s, false);
-          }
-          else
-          {
-            target.SetValue(Orthanc::DICOM_TAG_MODALITIES_IN_STUDY, "", false);
-            target.SetValue(Orthanc::DICOM_TAG_NUMBER_OF_STUDY_RELATED_SERIES, "0", false);
-            target.SetValue(Orthanc::DICOM_TAG_NUMBER_OF_STUDY_RELATED_INSTANCES, "0", false);
-          }
-
-          break;
-        }
-
-        case Orthanc::ResourceType_Series:
-        {
-          Json::Value series;
-          if (OrthancPlugins::RestApiGet(series, "/series/" + resource, false) &&
-              series.type() == Json::objectValue &&
-              series.isMember(INSTANCES) &&
-              series[INSTANCES].type() == Json::arrayValue)
-          {
-            if (series[INSTANCES].size() > 0 &&
-                series[INSTANCES][0].type() == Json::stringValue)
-            {
-              someInstance = series[INSTANCES][0].asString();
-            }
-            
-            // Number of Series Related Instances
-            target.SetValue(Orthanc::DICOM_TAG_NUMBER_OF_SERIES_RELATED_INSTANCES, 
-                            boost::lexical_cast<std::string>(series[INSTANCES].size()), false);
-          }
-          else
-          {
-            // Should never happen
-            target.SetValue(Orthanc::DICOM_TAG_NUMBER_OF_SERIES_RELATED_INSTANCES, "0", false);
-          }
-
-          break;
-        }
-
-        default:
-          throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
-      }
-    }                              
-
-
-    void ExtractFields(Orthanc::DicomMap& result,
-                       const Orthanc::DicomMap& source,
-                       const std::string& wadoBase,
-                       Orthanc::ResourceType level) const
-    {
-      std::set<Orthanc::DicomTag> fields;
-
       for (std::list<Orthanc::DicomTag>::const_iterator
              it = includeFields_.begin(); it != includeFields_.end(); ++it)
       {
@@ -447,19 +342,29 @@ namespace
         AddResultAttributesForLevel(fields, Orthanc::ResourceType_Series);
       }
 
+    }
+
+    void ExtractFields(Orthanc::DicomMap& result,
+                       const Orthanc::DicomMap& source,
+                       const std::string& wadoBasePublicUrl,
+                       Orthanc::ResourceType level) const
+    {
+      std::set<Orthanc::DicomTag> fields;
+      ExtractResultFields(fields, level);
+
       // Copy all the required fields to the target
       for (std::set<Orthanc::DicomTag>::const_iterator
              it = fields.begin(); it != fields.end(); ++it)
       {
-        std::string value;
-        if (source.LookupStringValue(value, *it, false /* no binary */))
+        if (source.HasTag(*it))
         {
-          result.SetValue(*it, value, false);
+          const Orthanc::DicomValue& val = source.GetValue(*it);
+          result.SetValue(*it, val);
         }
       }
 
       // Set the retrieve URL for WADO-RS
-      std::string url = (wadoBase + "studies/" +
+      std::string url = (wadoBasePublicUrl + "studies/" +
                          source.GetStringValue(Orthanc::DICOM_TAG_STUDY_INSTANCE_UID, "", false));
 
       if (level == Orthanc::ResourceType_Series || 
@@ -491,45 +396,36 @@ static void ApplyMatcher(OrthancPluginRestOutput* output,
 
   LOG(INFO) << "Body of the call from QIDO-RS to /tools/find: " << find.toStyledString();
   
-  std::string body;
-  OrthancPlugins::WriteFastJson(body, find);
-  
+  std::map<std::string, std::string> httpHeaders;
+  OrthancPlugins::GetHttpHeaders(httpHeaders, request);
+
   Json::Value resources;
-  if (!OrthancPlugins::RestApiPost(resources, "/tools/find", body, false) ||
+  if (!OrthancPlugins::RestApiPost(resources, "/tools/find", find, httpHeaders, true) ||
       resources.type() != Json::arrayValue)
   {
     throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
   }
 
-  std::string wadoBase = OrthancPlugins::Configuration::GetBaseUrl(request);
+  std::string wadoBasePublicUrl = OrthancPlugins::Configuration::GetBasePublicUrl(request);
 
   OrthancPlugins::DicomWebFormatter::HttpWriter writer(
     output, OrthancPlugins::Configuration::IsXmlExpected(request));
 
-  // Fix of issue #13
   for (Json::Value::ArrayIndex i = 0; i < resources.size(); i++)
   {
-    const std::string resource = resources[i].asString();
+    const Json::Value& resource = resources[i];
 
-    Orthanc::DicomMap derivedTags;
-    std::string someInstance;
-    matcher.ComputeDerivedTags(derivedTags, someInstance, level, resource);
-    
-    Json::Value tags;
-    if (!someInstance.empty() &&
-        OrthancPlugins::RestApiGet(tags, "/instances/" + someInstance + "/tags", false))
+    Orthanc::DicomMap source;
+    if (resource["RequestedTags"].isObject())
     {
-      Orthanc::DicomMap source;
-      source.FromDicomAsJson(tags);
-
-      Orthanc::DicomMap target;
-      target.Assign(derivedTags);
-      matcher.ExtractFields(target, source, wadoBase, level);
-
-      writer.AddOrthancMap(target);
+      source.FromDicomAsJson(resource["RequestedTags"], false, true);
     }
-  }
 
+    Orthanc::DicomMap target;
+
+    matcher.ExtractFields(target, source, wadoBasePublicUrl, level);
+    writer.AddOrthancMap(target);
+  }
   writer.Send();
 }
 
