@@ -3,7 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -24,7 +25,7 @@
 
 #include "Configuration.h"
 #include "DicomWebFormatter.h"
-
+#include "Logging.h"
 
 namespace OrthancPlugins
 {
@@ -116,7 +117,7 @@ namespace OrthancPlugins
     if (!ok)
     {
       // Bad DICOM file
-      LogWarning("STOW-RS cannot parse an incoming DICOM file");
+      LOG(WARNING) << "STOW-RS cannot parse an incoming DICOM file";
       hasBadSyntax_ = true;
       return;
     }
@@ -131,7 +132,7 @@ namespace OrthancPlugins
         dicom[Orthanc::DICOM_TAG_SOP_INSTANCE_UID.Format()].type() != Json::stringValue ||
         dicom[Orthanc::DICOM_TAG_STUDY_INSTANCE_UID.Format()].type() != Json::stringValue)
     {
-      LogWarning("STOW-RS: Missing a mandatory tag in incoming DICOM file");
+      LOG(WARNING) << "STOW-RS: Missing a mandatory tag in incoming DICOM file";
       hasBadSyntax_ = true;      
 
       if (dicom.isMember(Orthanc::DICOM_TAG_SOP_CLASS_UID.Format()) &&
@@ -162,8 +163,8 @@ namespace OrthancPlugins
     if (!expectedStudy_.empty() &&
         studyInstanceUid != expectedStudy_)
     {
-      LogWarning("STOW-RS request restricted to study [" + expectedStudy_ + 
-                 "], but received instance from study [" + studyInstanceUid + "]");
+      LOG(WARNING) << "STOW-RS request restricted to study [" << expectedStudy_ << 
+                      "], but received instance from study [" << studyInstanceUid << "]";
 
       hasConflict_ = true;
 
@@ -224,7 +225,7 @@ namespace OrthancPlugins
       }
       else
       {
-        LogError("Orthanc was unable to store one instance in a STOW-RS request");
+        LOG(ERROR) << "Orthanc was unable to store one instance in a STOW-RS request";
         item[DICOM_TAG_FAILURE_REASON.Format()] =
           boost::lexical_cast<std::string>(failureReason);
         failed_.append(item);
@@ -308,11 +309,11 @@ namespace OrthancPlugins
 
     if (expectedStudy.empty())
     {
-      LogInfo("STOW-RS request without study");
+      LOG(INFO) << "STOW-RS request without study";
     }
     else
     {
-      LogInfo("STOW-RS request restricted to study UID " + expectedStudy);
+      LOG(INFO) << "STOW-RS request restricted to study UID " << expectedStudy;
     }
 
     return new StowServer(context, headers, expectedStudy);
