@@ -3,7 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2021-2023 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -25,6 +26,7 @@
 #include "../Resources/Orthanc/Plugins/OrthancPluginCppWrapper.h"
 
 #include <Toolbox.h>
+#include <Logging.h>
 
 #include <memory>
 #include <list>
@@ -329,9 +331,18 @@ static const char* GetMimeType(const Orthanc::DicomTransferSyntax& syntax)
       // The "transfer-syntax" info was added in version 1.1 of the plugin
       return "application/octet-stream; transfer-syntax=1.2.840.10008.1.2";
 
+    // note: these 2 syntaxes are not supposed to be used in retrieve frames
+    // according to https://dicom.nema.org/MEDICAL/dicom/2019a/output/chtml/part18/chapter_6.html#table_6.1.1.8-3b
+    // "The Implicit VR Little Endian (1.2.840.10008.1.2), and Explicit VR Big Endian (1.2.840.10008.1.2.2) transfer syntaxes shall not be used with Web Services."
     case Orthanc::DicomTransferSyntax_LittleEndianExplicit:
       return "application/octet-stream; transfer-syntax=1.2.840.10008.1.2.1";
 
+    case Orthanc::DicomTransferSyntax_BigEndianExplicit:
+      return "application/octet-stream; transfer-syntax=1.2.840.10008.1.2.2";
+
+    case Orthanc::DicomTransferSyntax_DeflatedLittleEndianExplicit:
+      return "application/octet-stream; transfer-syntax=1.2.840.10008.1.2.1.99";
+
     case Orthanc::DicomTransferSyntax_JPEGProcess1:
       return "image/jpeg; transfer-syntax=1.2.840.10008.1.2.4.50";
 
@@ -369,7 +380,7 @@ static const char* GetMimeType(const Orthanc::DicomTransferSyntax& syntax)
       return "image/jpx; transfer-syntax=1.2.840.10008.1.2.4.93";
 
     default:
-      throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError, std::string("WADO RS Retrieve frame: unhandled Transfer syntax ") + Orthanc::GetTransferSyntaxUid(syntax));
   }
 }
 
@@ -437,7 +448,7 @@ static void RetrieveFrames(OrthancPluginRestOutput* output,
     { // logging only
       if (allFrames)
       {
-        OrthancPlugins::LogInfo("DICOMweb RetrieveFrames on " + orthancId + ", all frames");
+        LOG(INFO) << "DICOMweb RetrieveFrames on " << orthancId << ", all frames";
       }
       else
       {
@@ -448,7 +459,7 @@ static void RetrieveFrames(OrthancPluginRestOutput* output,
           s += boost::lexical_cast<std::string>(*frame + 1) + " ";
         }
 
-        OrthancPlugins::LogInfo(s);
+        LOG(INFO) << s;
       }
     }
 
@@ -463,7 +474,17 @@ static void RetrieveFrames(OrthancPluginRestOutput* output,
       throw Orthanc::OrthancException(Orthanc::ErrorCode_NotImplemented, "Unknown transfer syntax: " + currentSyntaxString);
     }
 
-    Orthanc::DicomTransferSyntax targetSyntax = currentSyntax;  
+    Orthanc::DicomTransferSyntax targetSyntax = currentSyntax;
+
+    if (currentSyntax == Orthanc::DicomTransferSyntax_BigEndianExplicit || currentSyntax == Orthanc::DicomTransferSyntax_LittleEndianImplicit)
+    {
+      // note: these 2 syntaxes are not supposed to be used in retrieve frames
+      // according to https://dicom.nema.org/MEDICAL/dicom/2019a/output/chtml/part18/chapter_6.html#table_6.1.1.8-3b
+      // "The Implicit VR Little Endian (1.2.840.10008.1.2), and Explicit VR Big Endian (1.2.840.10008.1.2.2) transfer syntaxes shall not be used with Web Services."
+      LOG(INFO) << "The file is in a transfer syntax " << currentSyntaxString << " that is not allowed by the DICOMWeb standard -> it will be transcoded to Little Endian Explicit";
+      targetSyntax = Orthanc::DicomTransferSyntax_LittleEndianExplicit;
+    }    
+
     bool transcodeThisInstance = false;
     
     if (ParseTransferSyntax(targetSyntax, request))
@@ -476,7 +497,7 @@ static void RetrieveFrames(OrthancPluginRestOutput* output,
     {
       if (!content.RestApiGet("/instances/" + orthancId + "/file?transcode=" + Orthanc::GetTransferSyntaxUid(targetSyntax), false))
       {
-        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError, "DICOMWeb: Unable to get transcoded file for for instance " + orthancId);
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError, "DICOMWeb: Unable to get transcoded file for instance " + orthancId);
       }
       instance.reset(new OrthancPlugins::DicomInstance(content.GetData(), content.GetSize()));
     }
@@ -484,14 +505,14 @@ static void RetrieveFrames(OrthancPluginRestOutput* output,
     {
       if (!content.RestApiGet("/instances/" + orthancId + "/file", false))
       {
-        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError, "DICOMWeb: Unable to get file for for instance " + orthancId);
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError, "DICOMWeb: Unable to get file for instance " + orthancId);
       }
       instance.reset(new OrthancPlugins::DicomInstance(content.GetData(), content.GetSize()));
 
       if (transcodeThisInstance)
       {
-        OrthancPlugins::LogInfo("DICOMweb RetrieveFrames: Transcoding instance " + orthancId + 
-                                " to transfer syntax " + Orthanc::GetTransferSyntaxUid(targetSyntax));
+        LOG(INFO) << "DICOMweb RetrieveFrames: Transcoding instance " + orthancId
+                  << " to transfer syntax " << Orthanc::GetTransferSyntaxUid(targetSyntax);
 
         instance.reset(OrthancPlugins::DicomInstance::Transcode(
                        content.GetData(), content.GetSize(), GetTransferSyntaxUid(targetSyntax)));
@@ -539,4 +560,4 @@ void RetrieveSelectedFrames(OrthancPluginRestOutput* output,
 void SetPluginCanDownloadTranscodedFile(bool enable)
 {
   pluginCanDownloadTranscodedFile = enable;
-}
\ No newline at end of file
+}
